package awais.instagrabber.activities;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.style.RelativeSizeSpan;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.SearchView;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.RecyclerView;

import awais.instagrabber.R;
import awais.instagrabber.adapters.CommentsAdapter;
import awais.instagrabber.asyncs.CommentsFetcher;
import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.interfaces.MentionClickListener;
import awais.instagrabber.models.CommentModel;
import awais.instagrabber.models.ProfileModel;
import awais.instagrabber.utils.Utils;

public class CommentsViewer extends AppCompatActivity {
    public static RecyclerView.RecycledViewPool recycledViewPool;
    private AlertDialog dialogComment, dialogMention;
    private CommentsAdapter commentsAdapter;
    private CommentModel commentModel;
    private View progressCircular;

    @Override
    protected void onCreate(final Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_comments);

        final Intent intent = getIntent();
        if (intent == null) {
            Utils.errorFinish(this);
            return;
        }

        final String shortCode = intent.getStringExtra("shortcode");
        if (Utils.isEmpty(shortCode)) {
            Utils.errorFinish(this);
            return;
        }

        final Toolbar toolbar = findViewById(R.id.toolbar);
        progressCircular = toolbar.findViewById(R.id.progress_circular);
        setSupportActionBar(toolbar);
        toolbar.setTitle("Comments");
        toolbar.setSubtitle(shortCode);

        final ArrayAdapter<String> commmentDialogAdapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1,
                new String[]{"Open profile", "View profile picture", "Copy username", "Copy comment"});
        final DialogInterface.OnClickListener profileDialogListener = (dialog, which) -> {
            final ProfileModel profileModel = commentModel.getProfileModel();

            if (which == 0) {
                searchUsername(profileModel.getUsername());
            } else if (which == 1) {
                startActivity(new Intent(this, ProfileViewer.class).putExtra("profile", profileModel));
            } else if (which == 2) {
                Utils.copyText(this, profileModel.getUsername());
            } else if (which == 3) {
                Utils.copyText(this, commentModel.getText().toString());
            }
        };

        final RecyclerView rvFollow = findViewById(R.id.rvComments);
        new CommentsFetcher(shortCode, new FetchListener<CommentModel[]>() {
            @Override
            public void doBefore() {
                progressCircular.setVisibility(View.VISIBLE);
            }

            @Override
            public void onResult(final CommentModel[] commentModels) {
                progressCircular.setVisibility(View.GONE);

                final View.OnClickListener clickListener = v -> {
                    final Object tag = v.getTag();
                    if (tag instanceof CommentModel) {
                        commentModel = (CommentModel) tag;
                        if (dialogComment == null)
                            dialogComment = new AlertDialog.Builder(CommentsViewer.this).setAdapter(commmentDialogAdapter, profileDialogListener)
                                    .setNeutralButton("Cancel", null).create();

                        final String username = commentModel.getProfileModel().getUsername();
                        final SpannableString title = new SpannableString(username + ":\n" + commentModel.getText());
                        title.setSpan(new RelativeSizeSpan(1.23f), 0, username.length(), Spanned.SPAN_INCLUSIVE_INCLUSIVE);
                        dialogComment.setTitle(title);

                        if (!dialogComment.isShowing()) dialogComment.show();
                    }
                };

                final MentionClickListener mentionClickListener = username -> {
                    if (dialogMention == null) dialogMention = new AlertDialog.Builder(CommentsViewer.this)
                            .setMessage("Do you want to search the username?").setNegativeButton("Cancel", null)
                            .setPositiveButton("OK", (dialog, which) -> CommentsViewer.this.searchUsername(username)).create();
                    dialogMention.setTitle(username);
                    if (!dialogMention.isShowing()) dialogMention.show();
                };

                if (recycledViewPool == null) {
                    recycledViewPool = new RecyclerView.RecycledViewPool();

                    if (commentModels != null) {
                        int size = commentModels.length;
                        for (final CommentModel commentModel : commentModels) {
                            if (commentModel != null) {
                                final CommentModel[] childCommentModels = commentModel.getChildCommentModels();
                                if (childCommentModels != null) size += childCommentModels.length;
                            }
                        }
                        if (size > 100) size >>>= 1;
                        recycledViewPool.setMaxRecycledViews(0, size);
                    }

                    rvFollow.setRecycledViewPool(recycledViewPool);
                }

                commentsAdapter = new CommentsAdapter(commentModels, true, clickListener, mentionClickListener);
                rvFollow.setAdapter(commentsAdapter);
            }
        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private void searchUsername(final String username) {
        if (Main.scanHack != null) {
            Main.scanHack.onResult(username);
            setResult(6969);
            finish();
        }
    }

    // todo prolly add comments search
    //  but it's gonna hang the whole ui thread if there are 500+ with lots of nested comments
    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        getMenuInflater().inflate(R.menu.follow, menu);

        final MenuItem menuSearch = menu.findItem(R.id.action_search);
        final SearchView searchView = (SearchView) menuSearch.getActionView();
        searchView.setQueryHint(getResources().getString(R.string.action_search));
        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(final String query) {
                return false;
            }

            @Override
            public boolean onQueryTextChange(final String query) {
                if (commentsAdapter != null) commentsAdapter.getFilter().filter(query);
                return true;
            }
        });

        menu.findItem(R.id.action_compare).setVisible(false);

        return true;
    }
}