package awais.instagrabber.activities;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.MatrixCursor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.PersistableBundle;
import android.provider.BaseColumns;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.ContentFrameLayout;
import androidx.appcompat.widget.SearchView;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.FragmentManager;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.google.android.material.appbar.AppBarLayout;

import java.util.ArrayList;
import java.util.Stack;

import awais.instagrabber.BuildConfig;
import awais.instagrabber.R;
import awais.instagrabber.adapters.SuggestionsAdapter;
import awais.instagrabber.asyncs.SuggestionsFetcher;
import awais.instagrabber.asyncs.SuggestionsFetcher.SuggestionType;
import awais.instagrabber.customviews.CircularImageView;
import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.models.ClipModel;
import awais.instagrabber.models.PostModel;
import awais.instagrabber.models.ProfileModel;
import awais.instagrabber.models.StoryModel;
import awais.instagrabber.models.SuggestionModel;
import awais.instagrabber.utils.FlavorTown;
import awais.instagrabber.utils.MainHelper;
import awais.instagrabber.utils.MyApps;
import awais.instagrabber.utils.Utils;
import awais.instagrabber.utils.dialogs.SettingsDialog;

import static awais.instagrabber.utils.Utils.sharedPreferences;
import static awais.instagrabber.utils.Utils.startClipboardMonitor;
import static awais.instagrabber.utils.dialogs.SettingConstants.BOTTOM_SNACK;

public final class Main extends AppCompatActivity {
    public static FetchListener<String> scanHack;
    public static final ArrayList<PostModel> allItems = new ArrayList<>();
    public final ArrayList<PostModel> selectedItems = new ArrayList<>();
    private AsyncTask<?, ?, ?> prevAsync;
    private SuggestionsAdapter suggestionAdapter;
    private MenuItem searchAction;
    public Toolbar toolbar;
    public SearchView searchView;
    public RecyclerView mainPosts;
    public MenuItem downloadAction;
    public StoryModel[] storyModels;
    public View privatePage, highlightsParent, isVerified;
    public SwipeRefreshLayout swipeRefreshLayout;
    public LinearLayout highlightsList;
    public CircularImageView mainProfileImage;
    public AppBarLayout appBarLayout;
    public TextView mainPostCount, mainFollowers, mainFollowing, mainFullName, mainBiography;
    public String userQuery = null;
    public MainHelper mainHelper;
    public ProfileModel profileModel;
    private AlertDialog profileClickDialog;
    private ArrayAdapter<String> profileDialogAdapter;
    private DialogInterface.OnClickListener profileDialogListener;
    private Stack<String> queriesStack;

    @Override
    protected void onCreate(final Bundle bundle) {
        super.onCreate(bundle);
        setContentView(R.layout.activity_main);

        FlavorTown.updateCheck(this);

        final String cookie = sharedPreferences.getString("cookie", null);
        Utils.setupCookies(cookie);

        if (Utils.isEmpty(cookie) || sharedPreferences.getBoolean("isFirstNew", true)) {
            new AlertDialog.Builder(this).setMessage("So... Instagram now needs you to login first to see posts and stuff, and it really sucks! :/")
                    .setPositiveButton("OK", (d, w) -> sharedPreferences.edit().putBoolean("isFirstNew", false).apply()).create();
        }

        MainHelper.stopCurrentExecutor();
        mainHelper = new MainHelper(this);
        if (bundle == null) {
            queriesStack = new Stack<>();
            userQuery = null;
        } else {
            setStack(bundle);
            userQuery = bundle.getString("query");
        }

        scanHack = result -> {
            if (mainHelper != null && !Utils.isEmpty(result)) {
                addToStack();
                userQuery = result;
                mainHelper.onRefresh();
            }
        };

        suggestionAdapter = new SuggestionsAdapter(this, v -> {
            final Object tag = v.getTag();
            if (tag instanceof CharSequence) {
                addToStack();
                userQuery = tag.toString();
                mainHelper.onRefresh();
            }
            if (searchView != null && !searchView.isIconified()) {
                if (searchAction != null) searchAction.collapseActionView();
                searchView.setIconified(true);
                searchView.setIconified(true);
            }
        });

        profileDialogAdapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, new String[]{"Show profile picture", "Show stories"});
        profileDialogListener = (dialog, which) -> {
            final Intent intent;
            if (which == 0 || storyModels == null || storyModels.length < 1)
                intent = new Intent(this, ProfileViewer.class).putExtra("profile", profileModel);
            else intent = new Intent(this, StoryViewer.class).putExtra("username", userQuery)
                    .putExtra("stories", storyModels);
            startActivity(intent);
        };
        initProfileDialog();

        final ContentFrameLayout parent = findViewById(android.R.id.content);
        final ViewGroup sliderView = (ViewGroup) LayoutInflater.from(this).inflate(R.layout.clip_slider, parent, false);
        final TextView sliderTextView = (TextView) sliderView.getChildAt(0);
        final int sliderHeight = Utils.convertDpToPx(64);
        final Handler handler = new Handler();
        final Runnable runnable = new Runnable() {
            @Override
            public void run() {
                sliderView.clearAnimation();
                sliderView.animate().translationY(sharedPreferences.getBoolean(BOTTOM_SNACK, true) ? sliderHeight : -sliderHeight).setListener(new AnimatorListenerAdapter() {
                    @Override
                    public void onAnimationEnd(final Animator animation) {
                        sliderView.clearAnimation();
                    }
                }).setDuration(700).start();
                handler.removeCallbacks(this);
            }
        };
        startClipboardMonitor(new FetchListener<ClipModel>() {
            private ClipModel clipModel;

            @Override
            public void onResult(final ClipModel model) {
                this.clipModel = model;

                if (model != null && parent != null && swipeRefreshLayout != null && !swipeRefreshLayout.isRefreshing()) {
                    final boolean bottomSnack = sharedPreferences.getBoolean(BOTTOM_SNACK, true);

                    runOnUiThread(() -> {
                        if (parent.getChildCount() == 1) {
                            if (bottomSnack)
                                sliderView.setLayoutParams(new FrameLayout.LayoutParams(-1, sliderHeight, Gravity.BOTTOM));
                            parent.addView(sliderView, parent.getChildCount());
                            sliderTextView.setOnClickListener(v -> {
                                handler.removeCallbacks(runnable);
                                sliderView.clearAnimation();
                                sliderView.setTranslationY(bottomSnack ? sliderHeight : -sliderHeight);
                                if (clipModel.isPost()) {
                                    startActivity(new Intent(Main.this, PostViewer.class)
                                            .putExtra("post", new PostModel(false, null, null,
                                                    null, clipModel.getText(), null, 0))
                                            .putExtra("user", userQuery));
                                } else {
                                    addToStack();
                                    userQuery = clipModel.getText();
                                    mainHelper.onRefresh();
                                }
                            });
                        }
                        sliderTextView.setText(clipModel.getInfoCaption());
                        sliderView.setTranslationY(bottomSnack ? sliderHeight : -sliderHeight);
                        sliderView.animate().translationY(0).setDuration(700).setListener(new AnimatorListenerAdapter() {
                            @Override
                            public void onAnimationEnd(final Animator animation) {
                                handler.postDelayed(runnable, 3017);
                            }
                        }).start();
                    });
                }
            }
        });

        final View.OnClickListener onClickListener = v -> {
            if (v == mainBiography) {
                Utils.copyText(this, mainBiography.getText().toString());
            } else if (v == mainProfileImage) {
                if (storyModels == null || storyModels.length <= 0) {
                    profileDialogListener.onClick(null, 0);
                } else {
                    if (profileClickDialog == null) initProfileDialog();
                    profileClickDialog.show();
                }
            }
        };

        mainBiography.setOnClickListener(onClickListener);
        mainProfileImage.setOnClickListener(onClickListener);

        mainBiography.setEnabled(false);
        mainProfileImage.setEnabled(false);

        final boolean isQueryNull = userQuery == null;
        if (isQueryNull) allItems.clear();
        if (BuildConfig.DEBUG && isQueryNull) userQuery = "the.badak"; // todo
        if (userQuery != null) mainHelper.onRefresh();

        mainHelper.onIntent(getIntent());
    }

    @Override
    protected void onNewIntent(final Intent intent) {
        super.onNewIntent(intent);
        mainHelper.onIntent(intent);
    }

    private void initProfileDialog() {
        if (profileClickDialog == null)
            profileClickDialog = new AlertDialog.Builder(this).setAdapter(profileDialogAdapter, profileDialogListener)
                    .setNeutralButton("Cancel", null).create();
    }

    @Override
    public void onSaveInstanceState(@NonNull final Bundle outState, @NonNull final PersistableBundle outPersistentState) {
        outState.putString("query", userQuery);
        outState.putSerializable("stack", queriesStack);
        super.onSaveInstanceState(outState, outPersistentState);
    }

    @Override
    public void onRestoreInstanceState(@Nullable final Bundle savedInstanceState, @Nullable final PersistableBundle persistentState) {
        super.onRestoreInstanceState(savedInstanceState, persistentState);
        if (savedInstanceState != null) {
            userQuery = savedInstanceState.getString("query");
            setStack(savedInstanceState);
        }
    }

    @Override
    protected void onSaveInstanceState(@NonNull final Bundle outState) {
        outState.putString("query", userQuery);
        outState.putSerializable("stack", queriesStack);
        super.onSaveInstanceState(outState);
    }

    @Override
    protected void onRestoreInstanceState(@NonNull final Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        userQuery = savedInstanceState.getString("query");
        setStack(savedInstanceState);
    }

    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        getMenuInflater().inflate(R.menu.menu, menu);

        final FragmentManager fragmentManager = getSupportFragmentManager();
        final MenuItem.OnMenuItemClickListener clickListener = item -> {
            if (item == downloadAction) Utils.batchDownload(this, userQuery, selectedItems);
            else new SettingsDialog().show(fragmentManager, "settings");
            return true;
        };

        menu.findItem(R.id.action_settings).setVisible(true).setOnMenuItemClickListener(clickListener);

        downloadAction = menu.findItem(R.id.action_download).setOnMenuItemClickListener(clickListener);

        searchAction = menu.findItem(R.id.action_search);
        searchView = (SearchView) searchAction.getActionView();

        searchView.setQueryHint(getResources().getString(R.string.action_search));
        searchView.setSuggestionsAdapter(suggestionAdapter);
        searchView.setOnSearchClickListener(v -> searchView.setQuery(userQuery, false));
        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            private final String[] COLUMNS = {BaseColumns._ID, "username", "type", "pfp", "verified"};

            @Override
            public boolean onQueryTextSubmit(final String query) {
                silentCloseSuggestionAsnyc();
                addToStack();
                userQuery = query;
                searchAction.collapseActionView();
                searchView.setIconified(true);
                searchView.setIconified(true);
                mainHelper.onRefresh();
                return false;
            }

            @Override
            public boolean onQueryTextChange(final String newText) {
                silentCloseSuggestionAsnyc();

                if (!Utils.isEmpty(newText) && newText.length() > 2) {
                    final boolean searchUser = newText.charAt(0) == '@';
                    final boolean searchHash = newText.charAt(0) == '#';

                    prevAsync = new SuggestionsFetcher(new FetchListener<SuggestionModel[]>() {
                        @Override
                        public void doBefore() {
                            suggestionAdapter.changeCursor(null);
                        }

                        @Override
                        public void onResult(final SuggestionModel[] result) {
                            final MatrixCursor cursor;
                            if (result == null) cursor = null;
                            else {
                                cursor = new MatrixCursor(COLUMNS, 0);
                                for (int i = 0; i < result.length; i++) {
                                    final SuggestionModel suggestionModel = result[i];
                                    if (suggestionModel != null) {
                                        final SuggestionType suggestionType = suggestionModel.getSuggestionType();
                                        final Object[] objects = {i, suggestionModel.getUsername(), suggestionType,
                                                suggestionModel.getProfilePic(), suggestionModel.isVerified()};

                                        if (!searchHash && !searchUser) cursor.addRow(objects);
                                        else {
                                            final boolean isCurrHash = suggestionType == SuggestionType.TYPE_HASHTAG;
                                            if (searchHash) {
                                                if (isCurrHash) cursor.addRow(objects);
                                            } else if (!isCurrHash) cursor.addRow(objects);
                                        }
                                    }
                                }

                            }
                            suggestionAdapter.changeCursor(cursor);
                        }
                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, searchUser || searchHash ? newText.substring(1) : newText);
                }
                return true;
            }
        });

        return true;
    }

    private void silentCloseSuggestionAsnyc() {
        if (prevAsync != null) try { prevAsync.cancel(true); } catch (final Exception ignored) { }
    }

    @Override
    public void onBackPressed() {
        if (searchView != null && !searchView.isIconified()) {
            if (searchAction != null) searchAction.collapseActionView();
            searchView.setIconified(true);
            searchView.setIconified(true);
            return;
        }

        if (!mainHelper.isSelectionCleared()) return;

        if (mainPosts != null) {
            final GridLayoutManager layoutManager = (GridLayoutManager) mainPosts.getLayoutManager();
            if (layoutManager != null && layoutManager.findFirstCompletelyVisibleItemPosition() >= layoutManager.getSpanCount()) {
                mainPosts.smoothScrollToPosition(0);
                appBarLayout.setExpanded(true, true);
                return;
            }
        }

        if (queriesStack != null && queriesStack.size() > 0) {
            userQuery = queriesStack.pop();
            if (userQuery != null) {
                mainHelper.onRefresh();
                return;
            }
        }

        MyApps.showAlertDialog(this, (parent, view, position, id) -> {
            if (id == -1 && position == -1 && parent == null) super.onBackPressed();
            else MyApps.openAppStore(this, position);
        });
    }

    @Override
    public void onRequestPermissionsResult(final int requestCode, @NonNull final String[] permissions, @NonNull final int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == 17 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            Utils.batchDownload(this, userQuery, selectedItems);
        }
    }

    @Override
    protected void onActivityResult(final int requestCode, final int resultCode, @Nullable final Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == 9629 && (resultCode == 1692 || resultCode == RESULT_CANCELED)) {
            finish();
        }
    }

    private void setStack(final Bundle bundle) {
        final Object stack = bundle != null ? bundle.get("stack") : null;
        if (stack instanceof Stack) //noinspection unchecked
            queriesStack = (Stack<String>) stack;
    }

    public void addToStack() {
        if (userQuery != null) {
            if (queriesStack == null) queriesStack = new Stack<>();
            queriesStack.add(userQuery);
        }
    }
}