package awais.instagrabber.activities;

import android.annotation.SuppressLint;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.appcompat.widget.LinearLayoutCompat;
import androidx.appcompat.widget.Toolbar;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.core.view.GestureDetectorCompat;

import com.github.chrisbanes.photoview.PhotoView;
import com.google.android.exoplayer2.Player;
import com.google.android.exoplayer2.SimpleExoPlayer;
import com.google.android.exoplayer2.source.MediaSource;
import com.google.android.exoplayer2.source.MediaSourceEventListener;
import com.google.android.exoplayer2.source.ProgressiveMediaSource;
import com.google.android.exoplayer2.ui.PlayerView;
import com.google.android.exoplayer2.upstream.DefaultDataSourceFactory;
import com.squareup.picasso.Callback;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import awais.instagrabber.R;
import awais.instagrabber.asyncs.DownloadAsync;
import awais.instagrabber.asyncs.PostFetcher;
import awais.instagrabber.interfaces.SwipeEvent;
import awais.instagrabber.models.BasePostModel;
import awais.instagrabber.models.PostModel;
import awais.instagrabber.models.ViewerPostModel;
import awais.instagrabber.utils.SwipeListener;
import awais.instagrabber.utils.Utils;
import awais.instagrabber.utils.dialogs.SettingConstants;

public class PostViewer extends AppCompatActivity {
    private String url;
    private PostModel postModel;
    private ViewerPostModel viewerPostModel;
    private SimpleExoPlayer player;
    private LinearLayoutCompat mediaList;
    private PlayerView playerView;
    private PhotoView photoView;
    private View progressView;
    private GestureDetectorCompat gestureDetector;
    private SwipeEvent swipeEvent;
    private AlertDialog alertDialog;
    private String postCaption = null, postShortCode;
    private ImageView btnMute;
    private TextView tvCommentsCount, tvVideoViews;
    private View btnDownload, btnComments, viewsContainer;
    private Toolbar toolbar;
    private boolean session = false, isFromShare;
    private int slidePos = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_viewer);

        toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        final Intent intent = getIntent();
        if (intent == null) {
            Utils.errorFinish(this);
            return;
        }

        postModel = (PostModel) intent.getSerializableExtra("post");
        if (postModel == null) {
            Utils.errorFinish(this);
            return;
        }

        postShortCode = postModel.getShortCode();

        postModel.setPosition(intent.getIntExtra("index", -1));

        if (postModel.getPostId() != null) {
            final String from = intent.getStringExtra("user");
            toolbar.setTitle((postModel.isVideo() ? "Video post" : "Image post") +
                    (from == null || Utils.isEmpty(from) ? "" : " from " + from));
        }

        isFromShare = postModel.getPosition() == -1 || postModel.getPostId() == null;

        btnDownload = findViewById(R.id.btnDownload);
        btnMute = findViewById(R.id.btnMute);
        btnComments = findViewById(R.id.btnComments);
        tvCommentsCount = btnComments.findViewById(R.id.commentsCount);
        tvVideoViews = findViewById(R.id.tvVideoViews);
        viewsContainer = (View) tvVideoViews.getParent();
        mediaList = findViewById(R.id.media_list);

        mediaList.setVisibility(View.GONE);
        swipeEvent = isRight -> {
            final boolean isMainSwipe = Main.allItems.size() >= 1 && !isFromShare;

            final int slides = mediaList != null ? mediaList.getChildCount() : 0;
            int position = postModel.getPosition();

            if (isRight) {
                --slidePos;
                if (!isMainSwipe && slidePos < 0) slidePos = 0;
                if (slides > 0 && slidePos >= 0) {
                    final View child = mediaList.getChildAt(slidePos);
                    if (child != null) child.performClick();
                    return;
                }
                if (isMainSwipe && --position < 0) position = Main.allItems.size() - 1;
            } else {
                ++slidePos;
                if (!isMainSwipe && slidePos >= slides) slidePos = slides - 1;
                if (slides > 0 && slidePos < slides) {
                    final View child = mediaList.getChildAt(slidePos);
                    if (child != null) child.performClick();
                    return;
                }
                if (isMainSwipe && ++position >= Main.allItems.size()) position = 0;
            }

            if (isMainSwipe) {
                slidePos = 0;
                Log.d("AWAISKING_APP", "swipe left <<< post[" + position + "]: " + postModel + " -- " + slides);
                postModel = Main.allItems.get(position);
                postModel.setPosition(position);
                viewPost();
            }
        };
        gestureDetector = new GestureDetectorCompat(this, new SwipeListener(swipeEvent));

        viewPost();
    }

    @SuppressLint("ClickableViewAccessibility")
    private void viewPost() {
        mediaList.setVisibility(View.GONE);
        mediaList.removeAllViews();
        btnDownload.setVisibility(View.INVISIBLE);
        btnMute.setVisibility(View.GONE);
        btnComments.setVisibility(View.GONE);
        viewsContainer.setVisibility(View.GONE);

        new PostFetcher(postModel.getShortCode(), result -> {
            if (result == null || result.length < 1) {
                Toast.makeText(this, "Unkown error occurred!!", Toast.LENGTH_SHORT).show();
                return;
            }

            final TextView viewerCaption = findViewById(R.id.viewerCaption);
            playerView = findViewById(R.id.playerView);
            progressView = findViewById(R.id.progressView);
            photoView = findViewById(R.id.imageViewer);

            final View.OnTouchListener gestureTouchListener = new View.OnTouchListener() {
                private float startX;
                private float startY;

                @Override
                public boolean onTouch(final View v, final MotionEvent event) {
                    if (v == viewerCaption) {
                        switch (event.getAction()) {
                            case MotionEvent.ACTION_DOWN:
                                startX = event.getX();
                                startY = event.getY();
                                break;

                            case MotionEvent.ACTION_UP:
                                if (!(Utils.isEmpty(postCaption) ||
                                        Math.abs(startX - event.getX()) > 50 || Math.abs(startY - event.getY()) > 50)) {
                                    Utils.copyText(PostViewer.this, postCaption);
                                    return false;
                                }
                        }
                    }
                    return gestureDetector.onTouchEvent(event);
                }
            };
            viewerCaption.setOnTouchListener(gestureTouchListener);
            playerView.setOnTouchListener(gestureTouchListener);
            photoView.setOnSingleFlingListener((e1, e2, velocityX, velocityY) -> {
                final float diffX = e2.getX() - e1.getX();
                if (Math.abs(diffX) > Math.abs(e2.getY() - e1.getY()) && Math.abs(diffX) > SwipeListener.SWIPE_THRESHOLD
                        && Math.abs(velocityX) > SwipeListener.SWIPE_VELOCITY_THRESHOLD) {
                    swipeEvent.onSwipe(diffX > 0);
                    return true;
                }
                return false;
            });

            if (result.length > 1) {
                final LayoutInflater layoutInflater = LayoutInflater.from(this);
                final View.OnClickListener clickListener = v -> {
                    final Object tag = v.getTag();
                    if (tag instanceof ViewerPostModel) {
                        slidePos = mediaList.indexOfChild(v);
                        viewerPostModel = (ViewerPostModel) tag;

                        postModel.setPostId(viewerPostModel.getPostId());
                        postModel.setTimestamp(viewerPostModel.getTimestamp());
                        postModel.setPostCaption(viewerPostModel.getPostCaption());

                        if (toolbar != null) {
                            final String username = viewerPostModel.getUsername();
                            toolbar.setTitle((viewerPostModel.isVideo() ? "Video post" : "Image post") +
                                    (username == null || Utils.isEmpty(username) ? "" : " from " + username));
                        }

                        refreshPost();
                    }
                };
                for (final BasePostModel model : result) {
                    final FrameLayout frameLayout = (FrameLayout) layoutInflater.inflate(R.layout.child_item, mediaList, false);
                    final AppCompatImageView imageView = (AppCompatImageView) frameLayout.getChildAt(0);

                    final String sliderDisplayUrl = model instanceof ViewerPostModel ? ((ViewerPostModel) model).getSliderDisplayUrl()
                            : model.getDisplayUrl();

                    Utils.PICASSO.load(sliderDisplayUrl).into(imageView);
                    frameLayout.setTag(model);
                    frameLayout.setOnClickListener(clickListener);
                    mediaList.addView(frameLayout);
                }
                mediaList.setVisibility(View.VISIBLE);
            }

            viewerPostModel = (ViewerPostModel) result[0];

            final long commentsCount = viewerPostModel.getCommentsCount();
            tvCommentsCount.setText(String.valueOf(commentsCount));
            btnComments.setVisibility(View.VISIBLE);

            btnComments.setOnClickListener(commentsCount > 0 ? v -> startActivityForResult(new Intent(PostViewer.this, CommentsViewer.class)
                    .putExtra("shortcode", postShortCode), 6969) : null);

            postModel.setPostId(viewerPostModel.getPostId());
            postModel.setTimestamp(viewerPostModel.getTimestamp());
            postModel.setPostCaption(viewerPostModel.getPostCaption());

            if (toolbar != null) {
                final String username = viewerPostModel.getUsername();
                toolbar.setTitle((viewerPostModel.isVideo() ? "Video post" : "Image post") +
                        (username == null || Utils.isEmpty(username) ? "" : " from " + username));
            }

            postCaption = postModel.getPostCaption();
            viewerCaption.setMovementMethod(new ScrollingMovementMethod());
            viewerCaption.setText(postCaption);

            btnDownload.setOnClickListener(v -> {
                if (ContextCompat.checkSelfPermission(this, Utils.PERMS[0]) == PackageManager.PERMISSION_GRANTED)
                    showDownloadDialog();
                else
                    ActivityCompat.requestPermissions(this, Utils.PERMS, 17);
            });

            refreshPost();
        }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private void setupVideo() {
        btnDownload.setVisibility(View.VISIBLE);
        btnMute.setVisibility(View.VISIBLE);
        playerView.setVisibility(View.VISIBLE);
        viewsContainer.setVisibility(View.VISIBLE);
        progressView.setVisibility(View.GONE);
        photoView.setVisibility(View.GONE);
        photoView.setImageDrawable(null);

        tvVideoViews.setText(String.valueOf(viewerPostModel.getVideoViews()));

        player = new SimpleExoPlayer.Builder(this).build();
        playerView.setPlayer(player);
        final float vol = Utils.sharedPreferences.getBoolean(SettingConstants.MUTED_VIDEOS, true) ? 0f : 1f;
        player.setVolume(vol);
        player.setPlayWhenReady(Utils.sharedPreferences.getBoolean(SettingConstants.AUTOPLAY_VIDEOS, true));
        final ProgressiveMediaSource mediaSource = new ProgressiveMediaSource.Factory(new DefaultDataSourceFactory(this, "instagram"))
                .createMediaSource(Uri.parse(url));
        mediaSource.addEventListener(new Handler(), new MediaSourceEventListener() {
            @Override
            public void onLoadCompleted(final int windowIndex, @Nullable final MediaSource.MediaPeriodId mediaPeriodId, final LoadEventInfo loadEventInfo, final MediaLoadData mediaLoadData) {
                progressView.setVisibility(View.GONE);
            }

            @Override
            public void onLoadStarted(final int windowIndex, @Nullable final MediaSource.MediaPeriodId mediaPeriodId, final LoadEventInfo loadEventInfo, final MediaLoadData mediaLoadData) {
                progressView.setVisibility(View.VISIBLE);
            }

            @Override
            public void onLoadCanceled(final int windowIndex, @Nullable final MediaSource.MediaPeriodId mediaPeriodId, final LoadEventInfo loadEventInfo, final MediaLoadData mediaLoadData) {
                progressView.setVisibility(View.GONE);
            }

            @Override
            public void onLoadError(final int windowIndex, @Nullable final MediaSource.MediaPeriodId mediaPeriodId, final LoadEventInfo loadEventInfo, final MediaLoadData mediaLoadData, final IOException error, final boolean wasCanceled) {
                progressView.setVisibility(View.GONE);
            }
        });
        player.prepare(mediaSource);

        player.setVolume(vol);

        btnMute.setImageResource(vol == 0f ? R.drawable.vol : R.drawable.mute);

        final View.OnClickListener onClickListener = v -> {
            if (player == null) return;
            if (v == playerView) {
                if (player.getPlaybackState() == Player.STATE_ENDED) player.seekTo(0);
                player.setPlayWhenReady(player.getPlaybackState() == Player.STATE_ENDED || !player.isPlaying());
            } else if (v == btnMute) {
                final float vol1 = player.getVolume() == 0f ? 1f : 0f;
                player.setVolume(vol1);
                btnMute.setImageResource(vol1 == 0f ? R.drawable.vol : R.drawable.mute);
            }
        };
        playerView.setOnClickListener(onClickListener);
        btnMute.setOnClickListener(onClickListener);
    }

    private void setupImage() {
        btnDownload.setVisibility(View.VISIBLE);
        btnMute.setVisibility(View.GONE);
        playerView.setVisibility(View.GONE);
        viewsContainer.setVisibility(View.GONE);
        progressView.setVisibility(View.VISIBLE);

        photoView.setImageDrawable(null);
        photoView.setVisibility(View.VISIBLE);
        photoView.setZoomable(true);
        photoView.setZoomTransitionDuration(417);
        photoView.setMaximumScale(7.17f);

        Utils.PICASSO.load(url).into(photoView, new Callback() {
            @Override
            public void onSuccess() {
                progressView.setVisibility(View.GONE);
            }

            @Override
            public void onError(final Exception e) {
                progressView.setVisibility(View.GONE);
            }
        });
    }

    private void downloadFile() {
        int error = 0;
        if (viewerPostModel != null) {
            final File dir = new File(Environment.getExternalStorageDirectory(), "Download");
            if (dir.exists() || dir.mkdirs()) {
                final String displayUrl = viewerPostModel.getDisplayUrl();
                final int index = displayUrl.indexOf('?');
                final File file = new File(dir, viewerPostModel.getPostId() + "_" + viewerPostModel.getTimestamp()
                        + displayUrl.substring(index - 4, index));

                new DownloadAsync(this, displayUrl, file, result -> {
                    if (result == null)
                        Toast.makeText(this, "Error downloading file", Toast.LENGTH_SHORT).show();
                    else if (result.exists())
                        Toast.makeText(this, "Post downloaded!!", Toast.LENGTH_SHORT).show();
                }).setItems(viewerPostModel.getShortCode(), null).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
            } else error = 1;

        } else error = 2;

        if (error == 1) Toast.makeText(this, "Error creating folder!", Toast.LENGTH_SHORT).show();
        else if (error == 2) Toast.makeText(this, "Unknown error occurred!!", Toast.LENGTH_SHORT).show();
    }

    private void showDownloadDialog() {
        final ArrayList<BasePostModel> postModels = new ArrayList<>();

        if (!session && mediaList.getVisibility() == View.VISIBLE) {
            if (alertDialog == null) {
                final DialogInterface.OnClickListener clickListener = (dialog, which) -> {
                    postModels.clear();

                    if (which == DialogInterface.BUTTON_NEGATIVE) {

                        for (int i = 0, size = mediaList.getChildCount(); i < size; ++i) {
                            final Object tag = mediaList.getChildAt(i).getTag();
                            if (tag instanceof ViewerPostModel) postModels.add((BasePostModel) tag);
                        }
                    } else if (which == DialogInterface.BUTTON_POSITIVE) {
                        postModels.add(viewerPostModel);
                    } else {
                        session = true;
                        postModels.add(viewerPostModel);
                    }

                    if (postModels.size() > 0) Utils.batchDownload(PostViewer.this, viewerPostModel.getUsername(), postModels);
                };
                alertDialog = new AlertDialog.Builder(this).setTitle("Select what to download")
                        .setMessage("Note: Current [Session] won't show this dialog until you're here, and will always download shown post slide.")
                        .setNeutralButton("Current [Session]", clickListener).setPositiveButton("Current", clickListener)
                        .setNegativeButton("Whole Album", clickListener).create();
            }
            if (!alertDialog.isShowing()) alertDialog.show();
        } else {
            Utils.batchDownload(PostViewer.this, viewerPostModel.getUsername(), Collections.singletonList(viewerPostModel));
        }
    }

    @Override
    public void onRequestPermissionsResult(final int requestCode, @NonNull final String[] permissions, @NonNull final int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == 17 && grantResults[0] == PackageManager.PERMISSION_GRANTED)
            showDownloadDialog();
    }

    @Override
    protected void onActivityResult(final int requestCode, final int resultCode, @Nullable final Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == 6969) {
            setResult(RESULT_OK);
            finish();
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        if (Build.VERSION.SDK_INT < 24) releasePlayer();
    }

    @Override
    public void onStop() {
        super.onStop();
        if (Build.VERSION.SDK_INT >= 24) releasePlayer();
    }

    private void refreshPost() {
        url = viewerPostModel.getDisplayUrl();
        releasePlayer();
        if (viewerPostModel.isVideo()) setupVideo();
        else setupImage();
    }

    private void releasePlayer() {
        if (player == null) return;
        player.release();
        player = null;
    }
}