package awais.instagrabber.activities;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.FragmentManager;

import com.github.chrisbanes.photoview.PhotoView;
import com.squareup.picasso.Callback;

import java.io.File;

import awais.instagrabber.R;
import awais.instagrabber.asyncs.DownloadAsync;
import awais.instagrabber.asyncs.ProfilePictureFetcher;
import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.models.ProfileModel;
import awais.instagrabber.utils.Utils;
import awais.instagrabber.utils.dialogs.ProfileSettingsDialog;
import awais.instagrabber.utils.dialogs.SettingConstants;

public class ProfileViewer extends AppCompatActivity {
    private ProfileModel profileModel;
    private MenuItem menuItemDownload;
    private String profilePicUrl;
    private FragmentManager fragmentManager;
    private FetchListener<String> fetchListener;
    private boolean errorHandled = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_profile);

        final Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        final Intent intent = getIntent();
        if (intent == null) {
            Utils.errorFinish(this);
            return;
        }

        profileModel = (ProfileModel) intent.getSerializableExtra("profile");
        if (profileModel == null) {
            Utils.errorFinish(this);
            return;
        }

        fragmentManager = getSupportFragmentManager();

        toolbar.setTitle(profileModel.getUsername());

        final View progressView = findViewById(R.id.progressView);
        final PhotoView photoView = findViewById(R.id.imageViewer);

        progressView.setVisibility(View.VISIBLE);
        photoView.setVisibility(View.VISIBLE);

        photoView.setZoomable(true);
        photoView.setZoomTransitionDuration(417);
        photoView.setMaximumScale(7.17f);

        fetchListener = profileUrl -> {
            profilePicUrl = profileUrl == null ? profileModel.getHdProfilePic() : profileUrl;
            Utils.PICASSO.load(profilePicUrl).into(photoView, new Callback() {
                @Override
                public void onSuccess() {
                    if (menuItemDownload != null) menuItemDownload.setEnabled(true);
                    progressView.setVisibility(View.GONE);
                }

                @Override
                public void onError(final Exception e) {
                    if (!errorHandled) {
                        errorHandled = true;
                        new ProfilePictureFetcher(profileModel.getUsername(), fetchListener, true).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                    }
                    progressView.setVisibility(View.GONE);
                }
            });
        };

        new ProfilePictureFetcher(profileModel.getUsername(), fetchListener,
                Utils.sharedPreferences.getBoolean(SettingConstants.USE_ALTER_PROFILE, false)).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        getMenuInflater().inflate(R.menu.menu, menu);

        final MenuItem.OnMenuItemClickListener menuItemClickListener = item -> {
            if (item == menuItemDownload) {
                if (profileModel == null) {
                    Toast.makeText(this, "Unknown error occurred!!", Toast.LENGTH_SHORT).show();
                    return true;
                }

                final File dir = new File(Environment.getExternalStorageDirectory(), "Download");
                if (!dir.exists() && !dir.mkdirs()) {
                    Toast.makeText(this, "Error creating folder!", Toast.LENGTH_SHORT).show();
                    return true;
                }

                final int index = profilePicUrl.indexOf('?');
                new DownloadAsync(this, profilePicUrl, new File(dir, profileModel.getUsername() + '_' +
                        System.currentTimeMillis() + profilePicUrl.substring(index - 4, index)), result -> {
                    if (result == null)
                        Toast.makeText(this, "Error downloading file", Toast.LENGTH_SHORT).show();
                    else if (result.exists())
                        Toast.makeText(this, "File downloaded in Downloads folder!", Toast.LENGTH_SHORT).show();
                }).setItems(null, profileModel.getUsername()).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
            } else {
                new ProfileSettingsDialog().show(fragmentManager, "settings");
            }
            return true;
        };

        menu.findItem(R.id.action_search).setVisible(false);
        menuItemDownload = menu.findItem(R.id.action_download);
        menuItemDownload.setVisible(true);
        menuItemDownload.setEnabled(false);
        menuItemDownload.setOnMenuItemClickListener(menuItemClickListener);

        final MenuItem menuItemSettings = menu.findItem(R.id.action_settings);
        menuItemSettings.setVisible(true);
        menuItemSettings.setOnMenuItemClickListener(menuItemClickListener);

        return true;
    }
}