package awais.instagrabber.asyncs;

import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.MediaMetadataRetriever;
import android.media.MediaScannerConnection;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Environment;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.core.app.NotificationCompat;
import androidx.core.content.FileProvider;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.concurrent.atomic.AtomicReference;

import awais.instagrabber.BuildConfig;
import awais.instagrabber.R;
import awais.instagrabber.activities.ProfileViewer;
import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.utils.Utils;

import static awais.instagrabber.utils.Utils.CHANNEL_ID;
import static awais.instagrabber.utils.Utils.CHANNEL_NAME;
import static awais.instagrabber.utils.Utils.isChannelCreated;
import static awais.instagrabber.utils.Utils.notificationManager;

public class DownloadAsync extends AsyncTask<Void, Float, File> {
    private static int lastNotifId = 0;
    private final int currentNotifId;
    private final AtomicReference<Context> context;
    private final File outFile;
    private final String url;
    private final FetchListener<File> fetchListener;
    private NotificationCompat.Builder downloadNotif;
    private String shortCode, username;

    public DownloadAsync(final Context context, final String url, final File outFile, final FetchListener<File> fetchListener) {
        this.context = new AtomicReference<>(context);
        this.url = url;
        this.outFile = outFile;
        this.fetchListener = fetchListener;
        this.shortCode = this.username = "Download started";
        this.currentNotifId = ++lastNotifId;
        if (++lastNotifId + 1 == Integer.MAX_VALUE) lastNotifId = 0;

        if (notificationManager == null)
            notificationManager = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);

        if (notificationManager != null) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O && !isChannelCreated) {
                notificationManager.createNotificationChannel(new NotificationChannel(CHANNEL_ID,
                        CHANNEL_NAME, NotificationManager.IMPORTANCE_HIGH));
                isChannelCreated = true;
            }

            downloadNotif = new NotificationCompat.Builder(context, CHANNEL_ID).setCategory(NotificationCompat.CATEGORY_STATUS)
                    .setSmallIcon(R.mipmap.ic_launcher).setContentText(shortCode == null ? username : shortCode)
                    .setProgress(100, 0, false)
                    .setContentTitle("Downloading " + (context instanceof ProfileViewer ? "profile picture" : "post"))
                    .setOngoing(true).setAutoCancel(false).setOnlyAlertOnce(true);

            notificationManager.notify(currentNotifId, downloadNotif.build());
        }
    }

    public DownloadAsync setItems(final String shortCode, final String username) {
        this.shortCode = shortCode;
        this.username = username;
        return this;
    }

    @Override
    protected File doInBackground(final Void... voids) {
        try {
            final URLConnection urlConnection = new URL(url).openConnection();
            final long fileSize = Build.VERSION.SDK_INT >= 24 ? urlConnection.getContentLengthLong() :
                    urlConnection.getContentLength();
            float totalRead = 0;

            try (final BufferedInputStream bis = new BufferedInputStream(urlConnection.getInputStream());
                 final FileOutputStream fos = new FileOutputStream(outFile)) {
                final byte[] buffer = new byte[8192];
                int count;
                while ((count = bis.read(buffer, 0, 8192)) != -1) {
                    totalRead = totalRead + count;
                    fos.write(buffer, 0, count);
                    publishProgress(totalRead * 100f / fileSize);
                }
                fos.flush();
            }

            return outFile;
        } catch (Exception e) {
            Log.e("AWAISKING_APP", "", e);
        }
        return null;
    }

    @Override
    protected void onPreExecute() {
        if (fetchListener != null) fetchListener.doBefore();
    }

    @Override
    protected void onProgressUpdate(@NonNull final Float... values) {
        if (downloadNotif != null) {
            downloadNotif.setProgress(100, values[0].intValue(), false);
            notificationManager.notify(currentNotifId, downloadNotif.build());
        }
    }

    @Override
    protected void onPostExecute(@NonNull final File result) {
        final Context context = this.context.get();
        Log.d("AWAISKING_APP", "preExec: " + lastNotifId + " -- " + currentNotifId + " -- " + context);

        context.sendBroadcast(Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT ?
                new Intent(Intent.ACTION_MEDIA_MOUNTED, Uri.fromFile(Environment.getExternalStorageDirectory())) :
                new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, Uri.fromFile(result.getAbsoluteFile()))
        );
        MediaScannerConnection.scanFile(context, new String[]{result.getAbsolutePath()}, null, null);

        if (notificationManager != null) {
            final Uri uri = FileProvider.getUriForFile(context, "awais.instagrabber.provider", result);

            final ContentResolver contentResolver = context.getContentResolver();
            Bitmap bitmap = null;
            if (Utils.isImage(uri, contentResolver)) {
                try {
                    bitmap = BitmapFactory.decodeStream(contentResolver.openInputStream(uri));
                } catch (Exception e) {
                    if (BuildConfig.DEBUG) Log.e("AWAISKING_APP", "", e);
                }
            }

            if (bitmap == null) {
                try (final MediaMetadataRetriever retriever = new MediaMetadataRetriever()) {
                    try {
                        retriever.setDataSource(context, uri);
                    } catch (Exception e) {
                        retriever.setDataSource(result.getAbsolutePath());
                    }
                    bitmap = retriever.getFrameAtTime();
                } catch (Exception e) {
                    if (BuildConfig.DEBUG) Log.e("AWAISKING_APP", "", e);
                }
            }

            downloadNotif.setContentText(null).setContentTitle("Download completed").setProgress(0, 0, false)
                    .setWhen(System.currentTimeMillis()).setOngoing(false).setOnlyAlertOnce(false).setAutoCancel(true)
                    .setContentIntent(PendingIntent.getActivity(context, 1717, new Intent(Intent.ACTION_VIEW, uri)
                            .addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_FROM_BACKGROUND | Intent.FLAG_GRANT_READ_URI_PERMISSION | Intent.FLAG_GRANT_WRITE_URI_PERMISSION)
                            .putExtra(Intent.EXTRA_STREAM, uri), PendingIntent.FLAG_UPDATE_CURRENT));

            if (bitmap != null)
                downloadNotif.setStyle(new NotificationCompat.BigPictureStyle().setBigContentTitle("Download completed").bigPicture(bitmap))
                        .setLargeIcon(bitmap).setBadgeIconType(NotificationCompat.BADGE_ICON_LARGE);

            Utils.notificationManager.cancel(currentNotifId);
            Utils.notificationManager.notify(currentNotifId + 1, downloadNotif.build());
        }

        if (fetchListener != null) fetchListener.onResult(result);
    }
}