package awais.instagrabber.asyncs;

import android.os.AsyncTask;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONObject;

import java.net.HttpURLConnection;
import java.net.URL;

import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.models.BasePostModel;
import awais.instagrabber.models.ViewerPostModel;
import awais.instagrabber.utils.Utils;

public class PostFetcher extends AsyncTask<Void, Void, BasePostModel[]> {
    private final String shortCode;
    private final FetchListener<BasePostModel[]> fetchListener;

    public PostFetcher(final String shortCode, final FetchListener<BasePostModel[]> fetchListener) {
        this.shortCode = shortCode;
        this.fetchListener = fetchListener;
    }

    @Override
    protected BasePostModel[] doInBackground(final Void... voids) {
        try {
            final HttpURLConnection conn = (HttpURLConnection) new URL("https://www.instagram.com/p/" + shortCode + "/?__a=1").openConnection();
            conn.setUseCaches(false);
            conn.connect();

            if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {
                final JSONObject media = new JSONObject(Utils.readFromConnection(conn)).getJSONObject("graphql")
                        .getJSONObject("shortcode_media");

                final JSONArray captions = media.getJSONObject("edge_media_to_caption").getJSONArray("edges");
                final boolean isVideo = media.has("is_video") && media.getBoolean("is_video");

                final String postCaption = captions.length() > 0 ? captions.getJSONObject(0).getJSONObject("node").getString("text") : null;
                final long timestamp = media.getLong("taken_at_timestamp");

                final String username = media.has("owner") ? media.getJSONObject("owner").getString("username") : null;

                final long commentsCount = media.has("edge_media_to_parent_comment") ? media.getJSONObject("edge_media_to_parent_comment").getLong("count") : 0;
                if (!media.has("edge_sidecar_to_children")) {
                    final ViewerPostModel viewerPostModel = new ViewerPostModel(isVideo, media.getString("id"),
                            isVideo ? media.getString("video_url") : Utils.getHighQualityImage(media),
                            shortCode, postCaption, username,
                            isVideo && media.has("video_view_count") ? media.getLong("video_view_count") : -1,
                            timestamp);
                    viewerPostModel.setCommentsCount(commentsCount);
                    return new ViewerPostModel[]{viewerPostModel};
                }

                final JSONArray children = media.getJSONObject("edge_sidecar_to_children").getJSONArray("edges");
                final ViewerPostModel[] postModels = new ViewerPostModel[children.length()];

                for (int i = 0; i < postModels.length; ++i) {
                    final JSONObject node = children.getJSONObject(i).getJSONObject("node");
                    final boolean isChildVideo = node.getBoolean("is_video");
                    postModels[i] = new ViewerPostModel(isChildVideo, node.getString("id"),
                            isChildVideo ? node.getString("video_url") : Utils.getHighQualityImage(node),
                            node.getString("shortcode"), postCaption, username,
                            isChildVideo && node.has("video_view_count") ? node.getLong("video_view_count") : -1,
                            timestamp);
                    postModels[i].setSliderDisplayUrl(node.getString("display_url"));
                }

                postModels[0].setCommentsCount(commentsCount);

//                Log.d("AWAISKING_APP", "node: " + node);
//                if (node.has("edge_media_to_parent_comment"))
//                    postModels[i].setCommentsCount(node.getJSONObject("edge_media_to_parent_comment").getLong("count"));

                return postModels;

            }
        } catch (Exception e) {
            Log.e("AWAISKING_APP", "", e);
        }
        return null;
    }

    @Override
    protected void onPostExecute(final BasePostModel[] postModels) {
        if (fetchListener != null) fetchListener.onResult(postModels);
    }
}
