package awais.instagrabber.asyncs;

import android.os.AsyncTask;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONObject;

import java.net.HttpURLConnection;
import java.net.URL;

import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.models.PostModel;
import awais.instagrabber.utils.Utils;

public class PostsFetcher extends AsyncTask<Void, Void, PostModel[]> {
    private final String endCursor;
    private final String id;
    private final FetchListener<PostModel[]> fetchListener;

    public PostsFetcher(final String id, final FetchListener<PostModel[]> fetchListener) {
        this.id = id;
        this.endCursor = "";
        this.fetchListener = fetchListener;
    }

    public PostsFetcher(final String id, final String endCursor, final FetchListener<PostModel[]> fetchListener) {
        this.id = id;
        this.endCursor = endCursor == null ? "" : endCursor;
        this.fetchListener = fetchListener;
    }

    @Override
    protected PostModel[] doInBackground(final Void... voids) {
        final boolean isHashTag = id.charAt(0) == '#';

        final String url;
        if (isHashTag)
            url = "https://www.instagram.com/graphql/query/?query_hash=ded47faa9a1aaded10161a2ff32abb6b&variables={\"tag_name\":\""
                    + id.substring(1) + "\",\"first\":150,\"after\":\"" + endCursor + "\"}";
        else
            url = "https://www.instagram.com/graphql/query/?query_id=17880160963012870&id=" + id + "&first=50&after=" + endCursor;

        try {
            final HttpURLConnection conn = (HttpURLConnection) new URL(url).openConnection();
            conn.setUseCaches(false);
            conn.connect();

            if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {
                final JSONObject mediaPosts = new JSONObject(Utils.readFromConnection(conn)).getJSONObject("data")
                        .getJSONObject(isHashTag ? "hashtag" : "user")
                        .getJSONObject(isHashTag ? "edge_hashtag_to_media" : "edge_owner_to_timeline_media");

                final String endCursor;
                final boolean hasNextPage;

                final JSONObject pageInfo = mediaPosts.getJSONObject("page_info");
                if (pageInfo.has("has_next_page")) {
                    hasNextPage = pageInfo.getBoolean("has_next_page");
                    endCursor = hasNextPage ? pageInfo.getString("end_cursor") : null;
                } else {
                    hasNextPage = false;
                    endCursor = null;
                }

                final JSONArray edges = mediaPosts.getJSONArray("edges");
                final PostModel[] models = new PostModel[edges.length()];
                for (int i = 0; i < models.length; ++i) {
                    final JSONObject mediaNode = edges.getJSONObject(i).getJSONObject("node");
                    final JSONArray captions = mediaNode.getJSONObject("edge_media_to_caption").getJSONArray("edges");

                    models[i] = new PostModel(mediaNode.getBoolean("is_video"), mediaNode.getString("id"),
                            mediaNode.getString("display_url"), mediaNode.getString("thumbnail_src"),
                            mediaNode.getString("shortcode"),
                            captions.length() > 0 ? captions.getJSONObject(0).getJSONObject("node").getString("text") : null,
                            mediaNode.getLong("taken_at_timestamp"));

                    models[i].setSlider(mediaNode.has("__typename") && mediaNode.getString("__typename").equals("GraphSidecar"));
                }

                if (models[models.length - 1] != null)
                    models[models.length - 1].setPageCursor(hasNextPage, endCursor);

                return models;
            }
        } catch (Exception e) {
            Log.e("AWAISKING_APP", "", e);
        }

        return null;
    }

    @Override
    protected void onPostExecute(final PostModel[] postModels) {
        if (fetchListener != null) fetchListener.onResult(postModels);
    }
}
