package awais.instagrabber.asyncs;

import android.os.AsyncTask;
import android.util.Log;

import androidx.annotation.Nullable;

import org.json.JSONArray;
import org.json.JSONObject;

import java.net.HttpURLConnection;
import java.net.URL;

import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.models.ProfileModel;
import awais.instagrabber.utils.Utils;

public final class ProfileFetcher extends AsyncTask<Void, Void, ProfileModel> {
    private final FetchListener<ProfileModel> fetchListener;
    private final String userName;

    public ProfileFetcher(String userName, FetchListener<ProfileModel> fetchListener) {
        this.userName = userName;
        this.fetchListener = fetchListener;
    }

    @Nullable
    @Override
    protected ProfileModel doInBackground(final Void... voids) {
        final String url = "https://www.instagram.com/" + userName + "/?__a=1";

        try {
            final HttpURLConnection conn = (HttpURLConnection) new URL(url).openConnection();
            conn.setUseCaches(true);
            conn.connect();

            if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {
                final JSONObject user = new JSONObject(Utils.readFromConnection(conn)).getJSONObject("graphql").getJSONObject("user");

                boolean isPrivate = user.getBoolean("is_private");
                final JSONObject timelineMedia = user.getJSONObject("edge_owner_to_timeline_media");
                if (timelineMedia.has("edges")) {
                    final JSONArray edges = timelineMedia.getJSONArray("edges");
                    if (edges.length() > 0) isPrivate = false;
                }

                return new ProfileModel(isPrivate, user.getBoolean("is_verified"),
                        user.getString("id"), userName, user.getString("full_name"),
                        user.getString("biography"), user.getString("profile_pic_url"),
                        user.getString("profile_pic_url_hd"),
                        timelineMedia.getLong("count"),
                        user.getJSONObject("edge_followed_by").getLong("count"),
                        user.getJSONObject("edge_follow").getLong("count"));
            }
        } catch (Exception e) {
            Log.e("AWAISKING_APP", "", e);
        }

        return null;
    }

    @Override
    protected void onPostExecute(final ProfileModel result) {
        if (fetchListener != null) fetchListener.onResult(result);
    }
}
