package awais.instagrabber.customviews;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.RectF;
import android.text.Layout;
import android.text.Selection;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.style.BackgroundColorSpan;
import android.text.style.ClickableSpan;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatTextView;

import awais.instagrabber.R;
import awais.instagrabber.interfaces.MentionClickListener;

public final class RamboTextView extends AppCompatTextView {
    private static final int highlightBackgroundSpanKey = R.id.tvComment;
    private static final RectF touchedLineBounds = new RectF();
    private ClickableSpan clickableSpanUnderTouchOnActionDown;
    private boolean isUrlHighlighted;
    private MentionClickListener mentionClickListener;

    public RamboTextView(final Context context) {
        super(context);
    }

    public RamboTextView(final Context context, final AttributeSet attrs) {
        super(context, attrs);
    }

    public RamboTextView(final Context context, final AttributeSet attrs, final int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    public void setMentionClickListener(final MentionClickListener mentionClickListener) {
        this.mentionClickListener = mentionClickListener;
    }

    @SuppressLint("ClickableViewAccessibility")
    @Override
    public boolean onTouchEvent(final MotionEvent event) {
        final boolean touchEvent = super.onTouchEvent(event);

        final CharSequence text = getText();
        if (text instanceof SpannableString) {
            final SpannableString spanText = (SpannableString) text;
            final ClickableSpan clickableSpanUnderTouch = findClickableSpanUnderTouch(this, spanText, event);

            final int action = event.getAction();
            if (action == MotionEvent.ACTION_DOWN) clickableSpanUnderTouchOnActionDown = clickableSpanUnderTouch;
            final boolean touchStartedOverAClickableSpan = clickableSpanUnderTouchOnActionDown != null;

            switch (action) {
                case MotionEvent.ACTION_DOWN:
                    if (clickableSpanUnderTouch != null) highlightUrl(clickableSpanUnderTouch, spanText);
                    return touchStartedOverAClickableSpan;

                case MotionEvent.ACTION_UP:
                    if (touchStartedOverAClickableSpan && clickableSpanUnderTouch == clickableSpanUnderTouchOnActionDown)
                        dispatchUrlClick(spanText, clickableSpanUnderTouch);
                    cleanupOnTouchUp(spanText);
                    return touchStartedOverAClickableSpan;

                case MotionEvent.ACTION_MOVE:
                    if (clickableSpanUnderTouch != null) highlightUrl(clickableSpanUnderTouch, spanText);
                    else removeUrlHighlightColor(spanText);
                    return touchStartedOverAClickableSpan;

                case MotionEvent.ACTION_CANCEL:
                    cleanupOnTouchUp(spanText);
                    return touchEvent;
            }
        }

        return touchEvent;
    }

    protected void dispatchUrlClick(final Spanned s, final ClickableSpan clickableSpan) {
        if (mentionClickListener != null) {
            mentionClickListener.onClick(s.subSequence(s.getSpanStart(clickableSpan) + 1, s.getSpanEnd(clickableSpan)).toString());
        }
    }

    protected void highlightUrl(final ClickableSpan clickableSpan, final SpannableString text) {
        if (!isUrlHighlighted) {
            isUrlHighlighted = true;

            final int spanStart = text.getSpanStart(clickableSpan);
            final int spanEnd = text.getSpanEnd(clickableSpan);
            final BackgroundColorSpan highlightSpan = new BackgroundColorSpan(getHighlightColor());
            text.setSpan(highlightSpan, spanStart, spanEnd, Spannable.SPAN_INCLUSIVE_INCLUSIVE);
            setTag(highlightBackgroundSpanKey, highlightSpan);
            Selection.setSelection(text, spanStart, spanEnd);
        }
    }

    protected void removeUrlHighlightColor(final SpannableString text) {
        if (isUrlHighlighted) {
            isUrlHighlighted = false;

            final BackgroundColorSpan highlightSpan = (BackgroundColorSpan) getTag(highlightBackgroundSpanKey);
            text.removeSpan(highlightSpan);

            Selection.removeSelection(text);
        }
    }

    private void cleanupOnTouchUp(final SpannableString text) {
        clickableSpanUnderTouchOnActionDown = null;
        removeUrlHighlightColor(text);
    }

    @Nullable
    private static ClickableSpan findClickableSpanUnderTouch(@NonNull final TextView textView, final SpannableString text, @NonNull final MotionEvent event) {
        final int touchX = (int) (event.getX() - textView.getTotalPaddingLeft() + textView.getScrollX());
        final int touchY = (int) (event.getY() - textView.getTotalPaddingTop() + textView.getScrollY());

        final Layout layout = textView.getLayout();
        final int touchedLine = layout.getLineForVertical(touchY);
        final int touchOffset = layout.getOffsetForHorizontal(touchedLine, touchX);

        touchedLineBounds.left = layout.getLineLeft(touchedLine);
        touchedLineBounds.top = layout.getLineTop(touchedLine);
        touchedLineBounds.right = layout.getLineWidth(touchedLine) + touchedLineBounds.left;
        touchedLineBounds.bottom = layout.getLineBottom(touchedLine);

        if (touchedLineBounds.contains(touchX, touchY)) {
            final Object[] spans = text.getSpans(touchOffset, touchOffset, ClickableSpan.class);
            for (final Object span : spans)
                if (span instanceof ClickableSpan) return (ClickableSpan) span;
        }

        return null;
    }
}