package awais.instagrabber.directdownload;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.Resources;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.WindowManager;
import android.widget.Toast;

import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import java.util.Arrays;

import awais.instagrabber.asyncs.PostFetcher;
import awais.instagrabber.models.ClipModel;
import awais.instagrabber.models.ViewerPostModel;
import awais.instagrabber.utils.Utils;

public final class DirectDownload extends Activity {
    private boolean isFound = false;
    private Intent intent;
    private Context context;

    @Override
    public void onWindowAttributesChanged(final WindowManager.LayoutParams params) {
        super.onWindowAttributesChanged(params);
        if (!isFound) {
            intent = getIntent();
            context = getApplicationContext();
            if (intent != null && context != null) {
                isFound = true;
                checkIntent();
            }
        }
    }

    @Override
    public Resources getResources() {
        if (!isFound) {
            intent = getIntent();
            context = getApplicationContext();
            if (intent != null && context != null) {
                isFound = true;
                checkIntent();
            }
        }
        return super.getResources();
    }

    private synchronized void checkIntent() {
        if (ContextCompat.checkSelfPermission(context, Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED)
            doDownload();
        else {
            final Handler handler = new Handler(Looper.getMainLooper());
            handler.post(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(context, "Please grant permissions and try downloading again!", Toast.LENGTH_LONG).show();
                    handler.removeCallbacks(this);
                }
            });
            ActivityCompat.requestPermissions(this, Utils.PERMS, 202);
        }
        finish();
    }

    private synchronized void doDownload() {
        final String action = intent.getAction();
        if (!Utils.isEmpty(action) && !"android.intent.action.MAIN".equals(action)) {
            boolean error = true;

            String data = null;
            final Bundle extras = intent.getExtras();
            if (extras != null) {
                final Object extraData = extras.get(Intent.EXTRA_TEXT);
                if (extraData != null) {
                    error = false;
                    data = extraData.toString();
                }
            }

            if (error) {
                final Uri intentData = intent.getData();
                if (intentData != null) data = intentData.toString();
            }

            if (data != null && !Utils.isEmpty(data)) {
                final ClipModel model = Utils.stripString(data);
                if (model != null && model.isPost()) {
                    final String text = model.getText();

                    new PostFetcher(text, result -> {
                        if (result != null && result.length > 0) {
                            if (result.length > 1) {
                                context.startActivity(new Intent(context, MultiDirectDialog.class)
                                        .setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS)
                                        .putExtra("posts", result));
                            } else {
//                                final ArrayList<PostModel> postModels = new ArrayList<>(1);
//                                final ViewerPostModel postModel = (ViewerPostModel) result[0];
//                                postModels.add(new PostModel(postModel.isVideo(), postModel.getPostId(), postModel.getDisplayUrl(),
//                                        postModel.getDisplayUrl(), postModel.getShortCode(), postModel.getPostCaption(), postModel.getTimestamp()));


                                final ViewerPostModel postModel = (ViewerPostModel) result[0];
                                Utils.batchDownload(context, postModel.getUsername(), Arrays.asList(result));
                            }
                        }
                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                }
            }
        }
    }
}