package awais.instagrabber.models;

import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.TextPaint;
import android.text.style.ClickableSpan;
import android.view.View;

import androidx.annotation.NonNull;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public final class CommentModel {
    private final static SimpleDateFormat DATETIME_PARSER;
    static {
        DATETIME_PARSER = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault());
    }
    private final ProfileModel profileModel;
    private final String id, dateTime;
    private final CharSequence text;
    private final long likes;
    private CommentModel[] childCommentModels;
    private boolean hasNextPage;
    private String endCursor;

    public CommentModel(final String id, final String text, final long timestamp, final long likes, final ProfileModel profileModel) {
        this.id = id;

        final boolean hasCommentUsername = text != null && text.contains("@");
        if (!hasCommentUsername) this.text = text;
        else {
            final int commentLength = text.length();
            final SpannableStringBuilder stringBuilder = new SpannableStringBuilder(text, 0, commentLength);

            for (int i = 0; i < commentLength; ++i) {
                char currChar = text.charAt(i);
                if (currChar == '@') {
                    final int startLen = i;
                    do {
                        if (++i == commentLength) break;
                        currChar = text.charAt(i);
                    } while (currChar != ' ');

//                    final String username = text.substring(startLen + 1, i);
                    stringBuilder.setSpan(new CommentMentionClickSpan(), startLen, i, Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
                }
            }

            this.text = stringBuilder;
        }

        this.likes = likes;
        this.profileModel = profileModel;
        this.dateTime = DATETIME_PARSER.format(new Date(timestamp * 1000L));
    }

    public String getId() {
        return id;
    }

    public CharSequence getText() {
        return text;
    }

    public String getDateTime() {
        return dateTime;
    }

    public long getLikes() {
        return likes;
    }

    public ProfileModel getProfileModel() {
        return profileModel;
    }

    public CommentModel[] getChildCommentModels() {
        return childCommentModels;
    }

    public void setChildCommentModels(final CommentModel[] childCommentModels) {
        this.childCommentModels = childCommentModels;
    }

    public void setPageCursor(final boolean hasNextPage, final String endCursor) {
        this.hasNextPage = hasNextPage;
        this.endCursor = endCursor;
    }

    public boolean hasNextPage() {
        return hasNextPage;
    }

    public String getEndCursor() {
        return endCursor;
    }

//    @NonNull
//    @Override
//    public String toString() {
//        try {
//            final JSONObject object = new JSONObject();
//            object.put("id", id);
//            object.put("text", text);
//            object.put("name", profileModel != null ? profileModel.getUsername() : "");
//            if (childCommentModels != null) object.put("childComments", childCommentModels);
//            return object.toString();
//        } catch (Exception e) {
//            return "{\"id\":\"" + id + "\", \"text\":\"" + text
//                     //(text != null ? text.replaceAll("\"", "\\\\\"") : "")
//                    + "\", \"name\":\"" + (profileModel != null ? profileModel.getUsername() : "") +
//                    (childCommentModels != null ? "\", \"childComments\":" + childCommentModels.length : "\"") + '}';
//        }
//    }

    private static final class CommentMentionClickSpan extends ClickableSpan {
        @Override
        public void onClick(@NonNull final View widget) { }

        @Override
        public void updateDrawState(@NonNull final TextPaint ds) {
            ds.setColor(ds.linkColor);
        }
    }
}