package awais.instagrabber.utils;

import android.content.Intent;
import android.content.res.ColorStateList;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.text.SpannableStringBuilder;
import android.text.style.RelativeSizeSpan;
import android.text.style.StyleSpan;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.appcompat.widget.AppCompatTextView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.google.android.material.appbar.AppBarLayout;
import com.google.android.material.shape.MaterialShapeDrawable;
import com.squareup.picasso.Callback;

import java.util.Arrays;

import awais.instagrabber.R;
import awais.instagrabber.activities.FollowViewer;
import awais.instagrabber.activities.Main;
import awais.instagrabber.activities.PostViewer;
import awais.instagrabber.activities.StoryViewer;
import awais.instagrabber.adapters.PostsAdapter;
import awais.instagrabber.asyncs.HighlightsFetcher;
import awais.instagrabber.asyncs.PostsFetcher;
import awais.instagrabber.asyncs.ProfileFetcher;
import awais.instagrabber.asyncs.StoryStatusFetcher;
import awais.instagrabber.interfaces.FetchListener;
import awais.instagrabber.models.BasePostModel;
import awais.instagrabber.models.ClipModel;
import awais.instagrabber.models.HighlightModel;
import awais.instagrabber.models.PostModel;

import static awais.instagrabber.utils.dialogs.SettingConstants.AUTOLOAD_POSTS;
import static awais.instagrabber.utils.dialogs.SettingConstants.BOTTOM_TOOLBAR;

public final class MainHelper implements SwipeRefreshLayout.OnRefreshListener {
    private static AsyncTask<?, ?, ?> currentlyExecuting;
    private final boolean autoloadPosts;
    private boolean hasNextPage = false;
    private String endCursor = null;
    private final FetchListener<PostModel[]> listener = new FetchListener<PostModel[]>() {
        @Override
        public void onResult(final PostModel[] result) {
            if (result != null) {
                final int oldSize = Main.allItems.size();
                Main.allItems.addAll(Arrays.asList(result));

                postsAdapter.notifyItemRangeInserted(oldSize, result.length);

                main.mainPosts.post(() -> main.mainPosts.setNestedScrollingEnabled(true));

                final String username = main.profileModel.getUsername();
                final String postFix = "/" + main.profileModel.getPostCount() + ')';

                main.toolbar.setTitle(isHashtag ? "Hashtag: " + main.userQuery : username + " (" + Main.allItems.size() + postFix);

                final PostModel model = result[result.length - 1];
                if (model != null) {
                    endCursor = model.getEndCursor();

                    if (endCursor == null && !isHashtag) {
                        main.toolbar.setTitle(username + " (" + main.profileModel.getPostCount() + postFix);
                        final Handler handler = new Handler();
                        handler.postDelayed(new Runnable() {
                            @Override
                            public void run() {
                                main.toolbar.setTitle(username);
                                handler.removeCallbacks(this);
                            }
                        }, 1017);
                    }

                    hasNextPage = model.hasNextPage();
                    if ((autoloadPosts && hasNextPage) && !isHashtag)
                        currentlyExecuting = new PostsFetcher(main.profileModel.getId(), endCursor, this)
                                .executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                    else
                        main.swipeRefreshLayout.setRefreshing(false);
                    model.setPageCursor(false, null);
                }
            }
        }
    };
    @NonNull
    private final Main main;
    private final View infoContainer, collapsingToolbar;
    private PostsAdapter postsAdapter;
    private boolean isHashtag;
    private RecyclerLazyLoader lazyLoader;

    public MainHelper(@NonNull final Main main) {
        stopCurrentExecutor();

        this.main = main;
        this.autoloadPosts = Utils.sharedPreferences.getBoolean(AUTOLOAD_POSTS, false);

        this.infoContainer = main.findViewById(R.id.infoContainer);
        main.mainProfileImage = infoContainer.findViewById(R.id.mainProfileImage);
        main.mainPostCount = infoContainer.findViewById(R.id.mainPostCount);
        main.mainFollowers = infoContainer.findViewById(R.id.mainFollowers);
        main.mainFollowing = infoContainer.findViewById(R.id.mainFollowing);
        main.isVerified = infoContainer.findViewById(R.id.isVerified);
        main.mainFullName = infoContainer.findViewById(R.id.mainFullName);
        main.mainBiography = infoContainer.findViewById(R.id.mainBiography);
        main.highlightsParent = infoContainer.findViewById(R.id.highlightsParent);
        main.highlightsList = main.highlightsParent.findViewById(R.id.highlightsList);
        main.toolbar = main.findViewById(R.id.toolbar);
        main.mainPosts = main.findViewById(R.id.mainPosts);
        main.privatePage = main.findViewById(R.id.privatePage);
        main.swipeRefreshLayout = main.findViewById(R.id.swipeRefreshLayout);
        main.swipeRefreshLayout.setOnRefreshListener(this);

        main.appBarLayout = main.findViewById(R.id.appBarLayout);
        collapsingToolbar = main.appBarLayout.getChildAt(0);
        final View profileInfo = infoContainer.findViewById(R.id.profileInfo);

        main.mainPosts.setNestedScrollingEnabled(false);

        final boolean isBottomToolbar = Utils.sharedPreferences.getBoolean(BOTTOM_TOOLBAR, true);

        //todo get color from appbarlayout
        int color = -1;
        final Drawable background = main.appBarLayout.getBackground();
        if (background instanceof MaterialShapeDrawable) {
            final MaterialShapeDrawable drawable = (MaterialShapeDrawable) background;
            final ColorStateList fillColor = drawable.getFillColor();
            if (fillColor != null) color = fillColor.getDefaultColor();
        } else {
            final Bitmap bitmap = Bitmap.createBitmap(9, 9, Bitmap.Config.ARGB_8888);
            final Canvas canvas = new Canvas();
            canvas.setBitmap(bitmap);
            background.draw(canvas);
            color = bitmap.getPixel(4, 4);
            if (!bitmap.isRecycled()) bitmap.recycle();
        }
        if (color == -1 || color == 0) color = main.getResources().getBoolean(R.bool.isNight) ? 0xff212121 : 0xfff5f5f5;
        profileInfo.setBackgroundColor(color);
        profileInfo.setClickable(true);
        if (!isBottomToolbar) main.toolbar.setBackgroundColor(color);

        main.appBarLayout.addOnOffsetChangedListener(new AppBarLayout.OnOffsetChangedListener() {
            private int height;

            @Override
            public void onOffsetChanged(final AppBarLayout appBarLayout, final int verticalOffset) {
                if (height == 0) {
                    height = profileInfo.getHeight();
                    collapsingToolbar.setMinimumHeight(height);
                }
                profileInfo.setTranslationY(-Math.min(0, verticalOffset));
            }
        });

        main.setSupportActionBar(main.toolbar);
        if (isBottomToolbar) {
            final LinearLayout linearLayout = (LinearLayout) main.toolbar.getParent();
            linearLayout.removeView(main.toolbar);
            linearLayout.addView(main.toolbar, linearLayout.getChildCount());
        }

        final GridAutofitLayoutManager layoutManager = new GridAutofitLayoutManager(main, Utils.convertDpToPx(130));
        main.mainPosts.setLayoutManager(layoutManager);
        main.mainPosts.addItemDecoration(new GridSpacingItemDecoration(Utils.convertDpToPx(4)));
        main.mainPosts.setAdapter(postsAdapter = new PostsAdapter(Main.allItems, v -> {
            final Object tag = v.getTag();
            if (tag instanceof PostModel) {
                final PostModel postModel = (PostModel) tag;

                if (postsAdapter.isSelecting) toggleSelection(postModel);
                else main.startActivity(new Intent(main, PostViewer.class).putExtra("index", postModel.getPosition())
                        .putExtra("post", postModel).putExtra("user", main.userQuery));
            }
        }, v -> { // long click listener
            final Object tag = v.getTag();
            if (tag instanceof PostModel) {
                postsAdapter.isSelecting = true;
                toggleSelection((PostModel) tag);
            }
            return true;
        }));

        if (!autoloadPosts) {
            this.lazyLoader = new RecyclerLazyLoader(layoutManager, (page, totalItemsCount) -> {
                if (hasNextPage && !isHashtag) {
                    main.swipeRefreshLayout.setRefreshing(true);
                    stopCurrentExecutor();
                    currentlyExecuting = new PostsFetcher(main.profileModel.getId(), endCursor, listener).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                    endCursor = null;
                }
            });
            main.mainPosts.addOnScrollListener(lazyLoader);
        }
    }

    public void onIntent(final Intent intent) {
        if (intent != null) {
            final String action = intent.getAction();
            if (!Utils.isEmpty(action) && !"android.intent.action.MAIN".equals(action)) {
                intent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION | Intent.FLAG_GRANT_READ_URI_PERMISSION);

                boolean error = true;

                String data = null;
                final Bundle extras = intent.getExtras();
                if (extras != null) {
                    final Object extraData = extras.get(Intent.EXTRA_TEXT);
                    if (extraData != null) {
                        error = false;
                        data = extraData.toString();
                    }
                }

                if (error) {
                    final Uri intentData = intent.getData();
                    if (intentData != null) data = intentData.toString();
                }

                if (data != null && !Utils.isEmpty(data)) {
                    if (data.indexOf('\n') > 0) data = data.substring(data.lastIndexOf('\n') + 1);

                    final ClipModel model = Utils.stripString(data);
                    if (model != null) {
                        final String text = model.getText();
                        if (model.isPost()) {
                            main.startActivityForResult(new Intent(main, PostViewer.class).putExtra("user", main.userQuery)
                                    .putExtra("post", new PostModel(false, null, null, null,
                                            text, null, 0)), 9629);
                        } else {
                            main.addToStack();
                            main.userQuery = text;
                            onRefresh();
                        }
                    }
                }
            }
        }
    }

    @Override
    public void onRefresh() {
        if (lazyLoader != null) lazyLoader.resetState();
        stopCurrentExecutor();
        Main.allItems.clear();
        main.selectedItems.clear();
        if (postsAdapter != null) {
            postsAdapter.isSelecting = false;
            postsAdapter.notifyDataSetChanged();
        }
        main.appBarLayout.setExpanded(true, true);
        main.privatePage.setVisibility(View.GONE);
        main.mainProfileImage.setImageBitmap(null);
        main.mainProfileImage.setImageDrawable(null);
        main.mainPostCount.setText(null);
        main.mainFollowers.setText(null);
        main.mainFollowing.setText(null);
        main.mainFullName.setText(null);
        main.mainBiography.setText(null);
        main.mainBiography.setEnabled(false);
        main.mainProfileImage.setEnabled(false);
        main.isVerified.setVisibility(View.GONE);

        main.mainPosts.setNestedScrollingEnabled(false);
        main.highlightsParent.setVisibility(View.GONE);
        collapsingToolbar.setVisibility(View.GONE);
        main.highlightsList.removeAllViews();

        main.swipeRefreshLayout.setRefreshing(main.userQuery != null);
        if (main.userQuery == null) {
            main.toolbar.setTitle(R.string.app_name);
            return;
        }

        isHashtag = main.userQuery.charAt(0) == '#';
        collapsingToolbar.setVisibility(isHashtag ? View.GONE : View.VISIBLE);
        main.toolbar.setTitle((isHashtag ? "Hashtag: " : "") + main.userQuery);

        if (isHashtag) {
            if (infoContainer != null) infoContainer.setVisibility(View.GONE);
            currentlyExecuting = new PostsFetcher(main.userQuery, listener).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
        } else {
            if (infoContainer != null) infoContainer.setVisibility(View.VISIBLE);
            new ProfileFetcher(main.userQuery, profileModel -> {
                main.profileModel = profileModel;

                if (profileModel == null) {
                    main.swipeRefreshLayout.setRefreshing(false);
                    Toast.makeText(main, "Error loading profile!\nTry logging in and search again.", Toast.LENGTH_SHORT).show();
                    main.toolbar.setTitle(R.string.app_name);
                    return;
                }

                main.isVerified.setVisibility(profileModel.isVerified() ? View.VISIBLE : View.GONE);
                final String profileId = profileModel.getId();

                final boolean isLoggedIn = !Utils.isEmpty(Utils.sharedPreferences.getString("cookie", null));
                if (isLoggedIn) {
                    new StoryStatusFetcher(profileId, result -> {
                        main.storyModels = result;
                        if (result != null && result.length > 0) main.mainProfileImage.setStoriesBorder();
                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);

                    new HighlightsFetcher(profileId, result -> {
                        if (result != null && result.length > 0) {
                            main.highlightsParent.setVisibility(View.VISIBLE);
                            final View.OnClickListener clickListener = v -> {
                                final Object tag = v.getTag();
                                if (tag instanceof HighlightModel) {
                                    final HighlightModel highlightModel = (HighlightModel) tag;
                                    main.startActivity(new Intent(main, StoryViewer.class).putExtra("username", main.userQuery)
                                            .putExtra("highlight", highlightModel.getTitle())
                                            .putExtra("stories", highlightModel.getStoryModels()));
                                }
                            };
                            final LayoutInflater layoutInflater = LayoutInflater.from(main);
                            for (final HighlightModel highlightModel : result) {
                                final ViewGroup frameLayout = (ViewGroup) layoutInflater.inflate(R.layout.highlight_item, main.highlightsList, false);
                                final AppCompatImageView imageView = (AppCompatImageView) frameLayout.getChildAt(0);
                                final AppCompatTextView textView = (AppCompatTextView) frameLayout.getChildAt(1);
                                Utils.PICASSO.load(highlightModel.getThumbnailUrl()).into(imageView);
                                textView.setText(highlightModel.getTitle());
                                frameLayout.setTag(highlightModel);
                                frameLayout.setOnClickListener(clickListener);
                                main.highlightsList.addView(frameLayout);
                            }
                        }
                    }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                }

                main.mainProfileImage.setEnabled(false);
                Utils.PICASSO.load(profileModel.getSdProfilePic()).into(main.mainProfileImage, new Callback() {
                    @Override
                    public void onSuccess() {
                        main.mainProfileImage.setEnabled(true);
                    }

                    @Override
                    public void onError(final Exception e) {
                        main.mainProfileImage.setEnabled(false);
                    }
                });

                final long followersCount = profileModel.getFollowersCount();
                final long followingCount = profileModel.getFollowingCount();

                final String postCount = String.valueOf(profileModel.getPostCount());
                SpannableStringBuilder span = new SpannableStringBuilder(postCount + "\nPosts");
                span.setSpan(new RelativeSizeSpan(1.2f), 0, postCount.length(), 0);
                span.setSpan(new StyleSpan(Typeface.BOLD), 0, postCount.length(), 0);
                main.mainPostCount.setText(span);

                final String followersCountStr = String.valueOf(followersCount);
                final int followersCountStrLen = followersCountStr.length();
                span = new SpannableStringBuilder(followersCountStr).append("\nFollowers");
                span.setSpan(new RelativeSizeSpan(1.2f), 0, followersCountStrLen, 0);
                span.setSpan(new StyleSpan(Typeface.BOLD), 0, followersCountStrLen, 0);
                main.mainFollowers.setText(span);

                final String followingCountStr = String.valueOf(followingCount);
                final int followingCountStrLen = followingCountStr.length();
                span = new SpannableStringBuilder(followingCountStr).append("\nFollowing");
                span.setSpan(new RelativeSizeSpan(1.2f), 0, followingCountStrLen, 0);
                span.setSpan(new StyleSpan(Typeface.BOLD), 0, followingCountStrLen, 0);
                main.mainFollowing.setText(span);

                main.mainFullName.setText(profileModel.getName());
                main.mainBiography.setText(profileModel.getBiography());
                main.mainFullName.setSelected(true);
                main.mainBiography.setEnabled(true);

                if (!profileModel.isPrivate()) {
                    main.swipeRefreshLayout.setRefreshing(true);
                    main.mainPosts.setVisibility(View.VISIBLE);
                    main.privatePage.setVisibility(View.GONE);

                    if (isLoggedIn) {
                        final View.OnClickListener followClickListener = v -> main.startActivity(new Intent(main, FollowViewer.class)
                                .putExtra("followers", v == main.mainFollowers)
                                .putExtra("name", profileModel.getUsername())
                                .putExtra("id", profileId));

                        main.mainFollowers.setOnClickListener(followersCount > 0 ? followClickListener : null);
                        main.mainFollowing.setOnClickListener(followingCount > 0 ? followClickListener : null);
                    }

                    currentlyExecuting = new PostsFetcher(profileId, listener).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
                } else {
                    main.swipeRefreshLayout.setRefreshing(false);
                    main.mainPosts.setVisibility(View.GONE);
                    main.privatePage.setVisibility(View.VISIBLE);
                }
            }).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
        }
    }

    public static void stopCurrentExecutor() {
        if (currentlyExecuting != null) {
            try {
                currentlyExecuting.cancel(true);
            } catch (final Exception e) {
                Log.e("AWAISKING_APP", "", e);
            }
        }
    }

    private void toggleSelection(final PostModel postModel) {
        if (postModel != null && postsAdapter != null) {
            if (postModel.isSelected()) main.selectedItems.remove(postModel);
            else main.selectedItems.add(postModel);
            postModel.setSelected(!postModel.isSelected());
            notifyAdapter(postModel);
        }
    }

    public void deselectSelection(final BasePostModel postModel) {
        if (postModel instanceof PostModel && postsAdapter != null) {
            main.selectedItems.remove(postModel);
            ((PostModel) postModel).setSelected(false);
            notifyAdapter((PostModel) postModel);
        }
    }

    private void notifyAdapter(final PostModel postModel) {
        if (main.selectedItems.size() < 1) postsAdapter.isSelecting = false;
        if (postModel.getPosition() < 0) postsAdapter.notifyDataSetChanged();
        else postsAdapter.notifyItemChanged(postModel.getPosition(), postModel);

        if (main.downloadAction != null) main.downloadAction.setVisible(postsAdapter.isSelecting);
    }

    public boolean isSelectionCleared() {
        if (postsAdapter != null && postsAdapter.isSelecting) {
            for (final PostModel postModel : main.selectedItems) postModel.setSelected(false);
            main.selectedItems.clear();
            postsAdapter.isSelecting = false;
            postsAdapter.notifyDataSetChanged();
            if (main.downloadAction != null) main.downloadAction.setVisible(false);
            return false;
        }
        return true;
    }
}
