package awais.instagrabber.utils;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import awais.instagrabber.interfaces.LazyLoadListener;

// thanks to nesquena's EndlessRecyclerViewScrollListener
//   https://gist.github.com/nesquena/d09dc68ff07e845cc622
public final class RecyclerLazyLoader extends RecyclerView.OnScrollListener {
    private int visibleThreshold = 5;       // The minimum amount of items to have below your current scroll position before loading more.
    private int currentPage = 0;            // The current offset index of data you have loaded
    private int previousTotalItemCount = 0; // The total number of items in the dataset after the last load
    private boolean loading = true;         // True if we are still waiting for the last set of data to load.
    private final LazyLoadListener lazyLoadListener;
    private final GridLayoutManager layoutManager;

    public RecyclerLazyLoader(@NonNull final GridLayoutManager layoutManager, final LazyLoadListener lazyLoadListener) {
        this.layoutManager = layoutManager;
        this.lazyLoadListener = lazyLoadListener;
        this.visibleThreshold = this.visibleThreshold * Math.max(3, layoutManager.getSpanCount());
    }

    @Override
    public void onScrolled(@NonNull final RecyclerView recyclerView, final int dx, final int dy) {
        final int totalItemCount = layoutManager.getItemCount();

        if (totalItemCount < previousTotalItemCount) {
            currentPage = 0;
            previousTotalItemCount = totalItemCount;
            if (totalItemCount == 0) loading = true;
        }

        if (loading && totalItemCount > previousTotalItemCount) {
            loading = false;
            previousTotalItemCount = totalItemCount;
        }

        if (!loading && layoutManager.findLastVisibleItemPosition() + visibleThreshold > totalItemCount) {
            if (lazyLoadListener != null) lazyLoadListener.onLoadMore(++currentPage, totalItemCount);
            loading = true;
        }
    }

    public void resetState() {
        this.currentPage = 0;
        this.previousTotalItemCount = 0;
        this.loading = true;
    }
}