package awais.instagrabber.utils.dialogs;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CompoundButton;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatCheckBox;

import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import awais.instagrabber.R;
import awais.instagrabber.activities.Login;
import awais.instagrabber.utils.Utils;

import static awais.instagrabber.utils.dialogs.SettingConstants.AUTOLOAD_POSTS;
import static awais.instagrabber.utils.dialogs.SettingConstants.AUTOPLAY_VIDEOS;
import static awais.instagrabber.utils.dialogs.SettingConstants.BOTTOM_SNACK;
import static awais.instagrabber.utils.dialogs.SettingConstants.BOTTOM_TOOLBAR;
import static awais.instagrabber.utils.dialogs.SettingConstants.DOWNLOAD_USER_FOLDER;
import static awais.instagrabber.utils.dialogs.SettingConstants.MUTED_VIDEOS;

public class SettingsDialog extends BottomSheetDialogFragment {
    private Activity activity;
    private final CompoundButton.OnCheckedChangeListener checkChangeListener = (checkBox, checked) -> {
        final SharedPreferences.Editor editor = Utils.sharedPreferences.edit();
        final int id = checkBox.getId();
        if (id == R.id.cbDownloadUsername) editor.putBoolean(DOWNLOAD_USER_FOLDER, checked);
        else if (id == R.id.cbBottomToolbar) editor.putBoolean(BOTTOM_TOOLBAR, checked);
        else if (id == R.id.cbBottomSnack) editor.putBoolean(BOTTOM_SNACK, checked);
        else if (id == R.id.cbAutoplayVideos) editor.putBoolean(AUTOPLAY_VIDEOS, checked);
        else if (id == R.id.cbMuteVideos) editor.putBoolean(MUTED_VIDEOS, checked);
        else if (id == R.id.cbAutoloadPosts) editor.putBoolean(AUTOLOAD_POSTS, checked);
        editor.apply();
    };
    private final View.OnClickListener clickListener = v -> {
        if (v instanceof Button) {
            startActivity(v.getId() == R.id.btnLogin ? new Intent(v.getContext(), Login.class) :
                    new Intent(Intent.ACTION_VIEW, Uri.parse("https://gitlab.com/AwaisKing/instagrabber/")));
        } else ((ViewGroup) v).getChildAt(0).performClick();
    };

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        final Dialog dialog = super.onCreateDialog(savedInstanceState);

        final Context context = getContext();
        activity = context instanceof Activity ? (Activity) context : getActivity();

        final View contentView = View.inflate(activity, R.layout.settings_dialog_modal, null);

        contentView.findViewById(R.id.btnLogin).setOnClickListener(clickListener);
        // i really hope nobody reads this
        contentView.findViewById(R.id.btnProjectileDysfunction).setOnClickListener(clickListener);

        final AppCompatCheckBox cbDownloadUsername = contentView.findViewById(R.id.cbDownloadUsername);
        final AppCompatCheckBox cbBottomToolbar = contentView.findViewById(R.id.cbBottomToolbar);
        final AppCompatCheckBox cbBottomSnack = contentView.findViewById(R.id.cbBottomSnack);
        final AppCompatCheckBox cbAutoplayVideos = contentView.findViewById(R.id.cbAutoplayVideos);
        final AppCompatCheckBox cbMuteVideos = contentView.findViewById(R.id.cbMuteVideos);
        final AppCompatCheckBox cbAutoloadPosts = contentView.findViewById(R.id.cbAutoloadPosts);

        cbDownloadUsername.setChecked(Utils.sharedPreferences.getBoolean(DOWNLOAD_USER_FOLDER, false));
        cbBottomToolbar.setChecked(Utils.sharedPreferences.getBoolean(BOTTOM_TOOLBAR, true));
        cbBottomSnack.setChecked(Utils.sharedPreferences.getBoolean(BOTTOM_SNACK, true));
        cbAutoplayVideos.setChecked(Utils.sharedPreferences.getBoolean(AUTOPLAY_VIDEOS, true));
        cbMuteVideos.setChecked(Utils.sharedPreferences.getBoolean(MUTED_VIDEOS, true));
        cbAutoloadPosts.setChecked(Utils.sharedPreferences.getBoolean(AUTOLOAD_POSTS, false));

        setupListener(cbDownloadUsername);
        setupListener(cbBottomToolbar);
        setupListener(cbBottomSnack);
        setupListener(cbAutoplayVideos);
        setupListener(cbMuteVideos);
        setupListener(cbAutoloadPosts);

        dialog.setContentView(contentView);

        return dialog;
    }

    @Override
    public void onDismiss(@NonNull final DialogInterface dialog) {
        super.onDismiss(dialog);
        if (activity != null) activity.recreate();
    }

    private void setupListener(@NonNull final AppCompatCheckBox checkBox) {
        checkBox.setOnCheckedChangeListener(checkChangeListener);
        ((View) checkBox.getParent()).setOnClickListener(clickListener);
    }
}